
# q-expansion of newform 912.2.bn.f, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 912
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[799, [1, 0]], [229, [1, 0]], [305, [-1, 0]], [97, [0, 1]]]
aps_data = [[0, 0], [1, 1], [2, 2], [-1, 0], [2, -4], [6, -3], [-2, -2], [3, 2], [0, 0], [0, -6], [-1, 2], [-3, 6], [-12, 12], [-1, 1], [-8, 4], [0, -12], [0, 0], [0, -7], [-10, 5], [6, -6], [7, -7], [-3, -3], [-6, 12], [0, 0], [4, 4], [12, -6], [5, -10], [18, 0], [0, 0], [-12, 0], [12, -6], [-2, -2], [-4, 2], [0, -5], [10, 10], [2, -4], [-7, 7], [11, 0], [0, -12], [6, -6], [-12, 0], [-8, 4], [10, -20], [-7, -7], [0, 0], [0, -25], [13, 13], [-11, -11], [-18, 0], [13, 0], [10, 10], [-14, 28], [-2, 1], [8, -4], [0, 0], [14, 14], [0, 0], [8, -8], [-2, 0], [0, -6], [-28, 28], [0, 0], [-18, -18], [-8, 16], [0, 14], [0, 6], [3, -6], [1, 1], [-14, -14], [-7, 0], [-6, 12], [4, 4], [0, 17], [4, -8], [7, -14], [-6, 6], [-32, 16], [-29, 29], [-24, 24], [40, -20], [-6, 12], [8, 8], [0, 12], [-22, 11], [5, 5], [28, -14], [30, 0], [17, 0], [4, 4], [19, 0], [-16, 32], [-12, 6], [2, -4], [6, 6], [31, -31], [36, -18], [0, 0], [18, 0], [6, -3], [0, 13], [-30, 15], [-4, 2], [-18, 0], [24, 0], [-5, 0], [-38, 0], [-4, -4], [-48, 24], [0, -30], [-1, 2], [19, -38], [38, -38], [44, -22], [17, 0], [0, 13], [24, -24], [5, -5], [-22, 44], [-10, 20], [0, 6], [-16, 8], [7, -14], [-36, 0], [30, 0], [-20, 0], [-6, -6], [0, 13], [-16, -16], [-37, 37], [-22, 0], [25, -25], [6, -6], [-42, 21], [-13, 13], [16, -32], [0, -37], [0, -24], [-15, 30], [42, 0], [-2, 4], [36, -18], [-28, 14], [0, 40], [0, 6], [29, -58], [54, -54], [-5, 5], [0, 0], [0, -35], [-24, 0], [3, 3], [-22, 44], [0, 13], [0, 0], [-14, -14], [12, 0], [53, 0], [30, 30], [0, -23], [0, -24], [18, 18], [-6, 6], [-29, 29], [-48, 48], [12, 0], [-42, 42], [-21, -21], [0, 53]]
